<?php
//-
// Copyright (c) 2011-2016 Joe Clarke <jclarke@cisco.com>
// All rights reserved.

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.

include_once 'db.inc.php';
include_once 'swreg/swreg.inc.php';
require_once 'Log.php';
require_once 'functions.php';

$dsn = "$db_driver:host=$db_host;dbname=$db_name";

$options = [
    PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION,
    PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_ASSOC,
    PDO::ATTR_EMULATE_PREPARES => false,
];

try {
    $dbh = new PDO($dsn, $db_user, $db_pass, $options);
} catch (PDOException $e) {
    die($e->getMessage());
}

$logger = Log::singleton('file', LOGFILE, TOOL_NAME.' : Add Logical Switch');
if ($logger === false) {
    die("Failed to open logfile.\n");
}
$mask = Log::UPTO(LOG_LEVEL);
$logger->setMask($mask);

$base = get_base($_SERVER['SCRIPT_NAME']);
$errors = array();

$ruser = '<AUTHENTICATION NOT ENABLED>';
if (isset($_SERVER['REMOTE_USER'])) {
    $ruser = $_SERVER['REMOTE_USER'];
}

if (isset($_REQUEST['submit'])) {
    $name = $_REQUEST['name'];
    $pid = $_REQUEST['pid'];
    $mdf = $_REQUEST['mdf'];
    $idf = $_REQUEST['idf'];
    $is_idf = (isset($_REQUEST['is_idf'])) ? $_REQUEST['is_idf'] : array();
    $ports = $_REQUEST['ports'];
    for ($i = 0; $i < count($name); ++$i) {
        $curr_idf = (isset($is_idf[$i])) ? $is_idf[$i] : 0;

        if ($name[$i] === null || $name[$i] == '') {
            continue;
        }
        if ($pid[$i] == '__BOGUS__') {
            array_push($errors, 'Please select a Product ID on row '.($i + 1));
        }

        if ($ports[$i] == '__BOGUS__') {
            array_push($errors, 'Please select ports required on row '.($i + 1));
        }
        if ($mdf[$i] == '__BOGUS__') {
            array_push($errors, 'Please select an MDF for row '.($i + 1));
        }

        if (ADDRESS_SCHEME == 'IDF' && $idf[$i] == '__BOGUS__' && $curr_idf == 0) {
            array_push($errors, 'Please select an IDF for row '.($i + 1));
        }

        $name[$i] = trim($name[$i]);

        if (count($errors) == 0) {
            $ip = null;
            $sql = '';
            if ((ADDRESS_SCHEME == 'IDF' && $curr_idf == 0) || ADDRESS_SCHEME == 'NEXT_FREE' || ADDRESS_SCHEME == 'DNS') {
                $params = array();
                if (ADDRESS_SCHEME == 'IDF') {
                    $sql = 'SELECT ip_address FROM SWITCHES WHERE name = ?';
                    $sth = null;
                    try {
                        $sth = $dbh->prepare($sql);
                        $sth->execute(array($idf[$i]));
                    } catch (PDOException $e) {
                        array_push($errors, 'Failed to get IDF IP for row '.($i + 1).": {$e->getMessage()}");
                    }
                    if ($sth !== null) {
                        $row = $sth->fetch();
                        $sth->closeCursor();

                        preg_match("/^(\d+\.\d+\.\d+\.)/", $row['ip_address'], $matches);
                        $sql = 'SELECT address FROM ADDRESSES WHERE location = ? AND address LIKE ? AND used = ? AND reserved = ? ORDER BY INET_ATON(address) LIMIT 1';
                        $params = array($mdf[$i], $matches[1].'%', '0', '0');
                    }
                } elseif (ADDRESS_SCHEME == 'NEXT_FREE') {
                    $sql = 'SELECT address FROM ADDRESSES WHERE location = ? AND used = ? AND reserved = ? ORDER BY INET_ATON(address) LIMIT 1';
                    $params = array($mdf[$i], '0', '0');
                } elseif (ADDRESS_SCHEME == 'DNS') {
                    $hostname = $name[$i].'.'.DNS_ZONE;
                    $ip = gethostbyname($hostname);
                    if ($ip == $hostname) {
                        $ip = gethostbyname($name[$i]);
                        if ($ip == $name[$i]) {
                            /* Don't make this fatal as we can fallback to manual
                             * allocation.
                             array_push($errors, "Unable to obtain IP address for either $hostame or {$name[$i]} on row " . ($i + 1));*/
                            $ip = null;
                        }
                    }
                    if ($ip !== null) {
                        $sql = 'SELECT address FROM ADDRESSES WHERE address = ?';
                        $params = array($ip);
                        try {
                            $sth = $dbh->prepare($sql);
                            $sth->execute($params);
                            $row = $sth->fetch();
                            $sth->closeCursor();
                            if (!$row['address']) {
                                array_push($errors, "IP address $ip is not valid in the ZTP system for {$name[$i]} at row ".($i + 1));
                            }
                            $ip = null;
                        } catch (PDOException $e) {
                            array_push($errors, 'Failed to validate IP for row '.($i + 1).": {$e->getMessage()}");
                        }
                    }
                }
                if (ADDRESS_SCHEME == 'IDF' || ADDRESS_SCHEME == 'NEXT_FREE') {
                    $sth = null;
                    try {
                        $sth = $dbh->prepare($sql);
                        $sth->execute($params);
                        $row = $sth->fetch();
                        $sth->closeCursor();
                        $ip = $row['address'];
                    } catch (PDOException $e) {
                        array_push($errors, 'Failed to get next IP for row '.($i + 1).": {$e->getMessage()}");
                    }
                }
            }
            if (count($errors) == 0) {
                $sql = 'REPLACE INTO SWITCHES (name, pid, is_idf, ip_address, ports_required) VALUES (?, ?, ?, ?, ?)';
                try {
                    $sth = $dbh->prepare($sql);
                    $sth->execute(array($name[$i], $pid[$i], $curr_idf, $ip, $ports[$i]));
                    $sth->closeCursor();
                    $logger->info("User {$ruser} added new logical switch: name={$name[$i]}, pid={$pid[$i]}, is_idf={$curr_idf}, ip_adress={$ip}, ports_required={$ports[$i]}");

                    if ($ip !== null) {
                        $sql = 'UPDATE ADDRESSES SET used=? WHERE address=?';
                        try {
                            $sth = $dbh->prepare($sql);
                            $sth->execute(array('1', $ip));
                            $sth->closeCursor();
                        } catch (PDOException $ie) {
                            array_push($errors, 'Failed to checkout IP for row '.($i + 1).": {$ie->getMessage()}");
                        }
                    }
                } catch (PDOException $e) {
                    array_push($errors, 'Failed to add new logical switch for row '.($i + 1).": {$e->getMessage()}");
                }
            }
        }
    }
    if (count($errors) == 0) {
        cleanup();
        header("Location: $base/logicsw.php");
        exit;
    }
}

$idfs = array();
foreach ($MDFS as $m) {
    $sql = 'SELECT s.name AS name FROM SWITCHES s, ADDRESSES a WHERE a.location = ? AND a.address = s.ip_address AND s.is_idf = ?';
    try {
        $sth = $dbh->prepare($sql);
        $sth->execute(array($m, '1'));
        $idfs[$m] = array();
        while ($row = $sth->fetch()) {
            array_push($idfs[$m], $row['name']);
        }
        $sth->closeCursor();
    } catch (PDOException $e) {
    }
}

print_header(TOOL_NAME.': Add Logical Switch');
?>
  <body class="flex">
<script language="Javascript">
var idfs = JSON.parse('<?=json_encode($idfs)?>');
function onMDFChange(mdf, row) {
	var midfs = idfs[mdf];
	var sel = document.getElementById('idf['+row+']');
	deleteOptions(sel);
	createOption(sel, '--Please Select--', '__BOGUS__');
	for (mi in midfs) {
		createOption(sel, midfs[mi], midfs[mi]);
	}
}

</script>
    <div id="headswreg">
        <div id="header">
	  <h1><?=TOOL_NAME?>: Add Logical Switch</h1>
        </div>
        <br/>
    </div>
    <div id="page">
      <div id="itemhead">
        <h2 class="tabeltitle">Logical Switches</h2>
       </div>
       <div align="center">
<?php
foreach ($errors as $error) {
    ?>
         <p class="error"><?=$error?></p>
<?php

}
?>
       </div>
       <form name="logical_switch_add_form" method="POST" action="<?=$_SERVER['PHP_SELF']?>">
         <table class="full" summary="New Logical Switch Table">
           <tr>
             <th>Row No.</th>
             <th>Hostname</th>
             <th>Product ID</th>
             <th>MDF</th>
             <th>Is IDF?</th>
             <th>IDF</th>
             <th>Ports Required</th>
           </tr>
<?php
for ($i = 0; $i < 10; ++$i) {
    ?>
           <tr <?=get_row_color($i)?>>
             <td><?=$i + 1?>.</td>
             <td><input type="text" size="16" name="name[<?=$i?>]"></td>
	     <td><select name="pid[<?=$i?>]">
               <option value="__BOGUS__" selected>--Please Select--</option>
<?php
    foreach ($ZTP_PIDS as $p) {
        ?>
	       <option value="<?=$p?>"><?=$p?></option>
<?php

    } ?>
             </select></td>
	     <td><select name="mdf[<?=$i?>]" onChange='onMDFChange(this.options[this.selectedIndex].value, <?=$i?>); return true;'>
               <option value="__BOGUS__" selected>--Please Select--</option>
<?php
    foreach ($MDFS as $m) {
        ?>
               <option value="<?=$m?>"><?=$m?></option>
<?php

    } ?>
             </select></td>
	     <td><input type="checkbox" name="is_idf[<?=$i?>]" value="1" onClick="if (this.checked == true) { this.document.getElementById('idf[<?=$i?>]').disabled = true; } else { this.document.getElementById('idf[<?=$i?>]').disabled = false; }"></td>
             <td><select name="idf[<?=$i?>]" id="idf[<?=$i?>]">
               <option value="__BOGUS__" selected>--Please Select--</option>
             </select></td>
	     <td><select name="ports[<?=$i?>]" id="ports[<?=$i?>]">
               <option value="__BOGUS__" selected>--Please Select--</option>
<?php
    $nports = array_values($PID_PORTS);
    sort($nports);
    foreach ($nports as $n) {
        ?>
               <option value="<?=$n?>"><?=$n?></option>
<?php

    } ?>
             </select></td>
           </tr>
<?php

}
?>
         </table>
         <br/>
         <div align="center">
           <input type="reset" name="reset" value="Reset">
           <input type="submit" name="submit" value="Add">
         </div>
       </form>
       <table class="noborder" summary="Return Table">
         <tr>
           <td class="left_act">
             <a href="<?=$base?>/logicsw.php">&lt;&lt;&lt;Return to Main Page</a>
           </td>
         </tr>
       </table>
     </div>
   </body>
</html>
<?php
cleanup();
?>
