<?php
//-
// Copyright (c) 2011-2016 Joe Clarke <jclarke@cisco.com>
// All rights reserved.

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.

include_once 'db.inc.php';
include_once 'swreg/swreg.inc.php';
require_once 'Log.php';
require_once 'functions.php';

$dsn = "$db_driver:host=$db_host;dbname=$db_name";

$options = [
    PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION,
    PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_ASSOC,
    PDO::ATTR_EMULATE_PREPARES => false,
];

try {
    $dbh = new PDO($dsn, $db_user, $db_pass, $options);
} catch (PDOException $e) {
    die($e->getMessage());
}

$logger = Log::singleton('file', LOGFILE, TOOL_NAME.' : Add Physical Switch');
if ($logger === false) {
    die("Failed to open logfile.\n");
}
$mask = Log::UPTO(LOG_LEVEL);
$logger->setMask($mask);

$base = get_base($_SERVER['SCRIPT_NAME']);
$errors = array();

$ruser = '<AUTHENTICATION NOT ENABLED>';
if (isset($_SERVER['REMOTE_USER'])) {
    $ruser = $_SERVER['REMOTE_USER'];
}

if (isset($_REQUEST['submit'])) {
    $sn = $_REQUEST['sn'];
    $pid = $_REQUEST['pid'];
    $ports = $_REQUEST['ports'];
    $mac = $_REQUEST['mac'];
    for ($i = 0; $i < count($sn); ++$i) {
        if ($sn[$i] === null || $sn[$i] == '') {
            continue;
        }
        foreach ($PID_PORTS as $pp => $np) {
            if ($pid[$i] == $pp) {
                if ($ports[$i] != $np) {
                    array_push($errors, 'A '.$pp.' can only have '.$np.' ports on row '.($i + 1));
                }
                break;
            }
        }

        if ($pid[$i] == '__BOGUS__') {
            array_push($errors, 'Please select a Product ID on row '.($i + 1));
        }

        if ($ports[$i] == '__BOGUS__') {
            array_push($errors, 'Please select number of ports on row '.($i + 1));
        }

        $sn[$i] = trim($sn[$i]);

        if (count($errors) == 0) {
            $sql = 'REPLACE INTO DEVICE_MAP (serial_number, pid, mac, max_ports) VALUES (?, ?, ?, ?)';
            try {
                $sth = $dbh->prepare($sql);
                if ($mac[$i] === null || $mac[$i] == '') {
                    $mac[$i] = ' ';
                }
                $sth->execute(array($sn[$i], $pid[$i], $mac[$i], $ports[$i]));
                $sth->closeCursor();
                $logger->info("User {$ruser} added physical switch: serial_number={$sn[$i]}, pid={$pid[$i]}, mac={$mac[$i]}, max_ports={$ports[$i]}");
            } catch (PDOException $e) {
                array_push($errors, 'Failed to add new physical switch at row '.($i + 1).": {$e->getMessage()}");
            }
        }
    }
    if (count($errors) == 0) {
        cleanup();
        header("Location: $base/index.php");
        exit;
    }
}

print_header(TOOL_NAME.': Add Physical Switch');
?>
  <body class="flex">
    <div id="headswreg">
        <div id="header">
	  <h1><?=TOOL_NAME?>: Add Physical Switch</h1>
        </div>
        <br/>
    </div>
    <div id="page">
      <div id="itemhead">
        <h2 class="tabeltitle">Physical Switches</h2>
       </div>
       <div align="center">
<?php
foreach ($errors as $error) {
    ?>
         <p class="error"><?=$error?></p>
<?php

}
?>
       </div>
       <form name="phys_switch_add_form" method="POST" action="<?=$_SERVER['PHP_SELF']?>">
         <table class="full" summary="New Physical Switch Table">
           <tr>
             <th>Row No.</th>
             <th>Serial Number</th>
             <th>Product ID</th>
             <th>MAC Address</th>
             <th>Number of Ports</th>
           </tr>
<?php
for ($i = 0; $i < 10; ++$i) {
    ?>
           <tr <?=get_row_color($i)?>>
             <td><?=$i + 1?>.</td>
             <td><input type="text" size="16" name="sn[<?=$i?>]"></td>
             <td><select name="pid[<?=$i?>]">
               <option value="__BOGUS__" selected>--Please Select--</option>
<?php
    foreach ($ZTP_PIDS as $p) {
        ?>
	       <option value="<?=$p?>"><?=$p?></option>
<?php

    } ?>
             </select></td>
             <td><input type="text" name="mac[<?=$i?>]" size="16"></td>
             <td><select name="ports[<?=$i?>]">
               <option value="__BOGUS__" selected>--Please Select--</option>
<?php
    $nports = array_values($PID_PORTS);
    sort($nports);
    foreach ($nports as $np) {
        ?>
               <option value="<?=$np?>"><?=$np?></option>
<?php

    } ?>
             </select></td>
           </tr>
<?php

}
?>
         </table>
         <br/>
         <div align="center">
           <input type="reset" name="reset" value="Reset">
           <input type="submit" name="submit" value="Add">
         </div>
       </form>
       <table class="noborder" summary="Return Table">
         <tr>
           <td class="left_act">
             <a href="<?=$base?>/index.php">&lt;&lt;&lt;Return to Main Page</a>
           </td>
         </tr>
       </table>
     </div>
   </body>
</html>
<?php
cleanup();
?>
