#!/usr/bin/env -S php
<?php
//-
// Copyright (c) 2011-2016 Joe Clarke <jclarke@cisco.com>
// All rights reserved.

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.

include_once '../db.inc.php';
include_once 'swreg.inc.php';
require_once 'Log.php';

$dsn = "$db_driver:host=$db_host;dbname=$db_name";

$options = [
    PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION,
    PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_ASSOC,
    PDO::ATTR_EMULATE_PREPARES => false,
];

try {
    $dbh = new PDO($dsn, $db_user, $db_pass, $options);
} catch (PDOException $e) {
    die($e->getMessage());
}

$logger = Log::singleton('file', LOGFILE, TOOL_NAME.' : Pinger');
if ($logger === false) {
    die("Failed to open logfile.\n");
}
$mask = Log::UPTO(LOG_LEVEL);
$logger->setMask($mask);

$FPING = FPING_PROGRAM;

$sql = 'SELECT ps.serial_number AS serial_number, ls.ip_address AS ip_address, ps.device_status AS status FROM DEVICE_MAP ps, SWITCHES ls WHERE ps.assigned_switch = ls.name AND ps.provisioned_status=?';
$sth = null;
try {
    $sth = $dbh->prepare($sql);
    $sth->execute(array(PROVISION_SUCCESS));
} catch (PDOException $e) {
    $logger->crit("Failed get list of provisioned devices: {$e->getMessage()}");
    exit(1);
}

while ($sth !== null && $row = $sth->fetch()) {
    $status = REACHABILITY_NEVER_REACHABLE;
    $output = null;
    $result = 0;
    exec("$FPING -q -C 1 {$row['ip_address']} 2>/dev/null", $output, $result);
    exec("$FPING -q -C 1 {$row['ip_address']} 2>/dev/null", $output, $result);
    if ($result == 0) {
        $status = REACHABILITY_REACHABLE;
    } else {
        if ($row['status'] == REACHABILITY_REACHABLE || $row['status'] == REACHABILITY_NOW_UNREACHABLE) {
            $status = REACHABILITY_NOW_UNREACHABLE;
        }
    }

    $sql = 'UPDATE DEVICE_MAP SET device_status = ? WHERE serial_number = ?';
    try {
        $isth = $dbh->prepare($sql);
        $isth->execute(array($status, $row['serial_number']));
        $isth->closeCursor();
    } catch (PDOException $e) {
        $logger->error("Failed to update switch status : {$e->getMessage()}");
        continue;
    }
}

$sth->closeCursor();

$logger->close();

?>
